.ui-icon-nag-unchecked {
    background-image: none;
}
.overdue a, span.overdue {
    color: #f00 !important;
}

.closed a {
    color: #aaa !important;
    text-decoration: line-through !important;
}

#nag-notasks {
    font-style: italic;
    color: #888;
    text-align: center;
}
